package com.seeyon.v3x.edoc.dao;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.seeyon.ctp.util.FlipInfo;
import com.seeyon.ctp.util.ParamUtil;
import com.seeyon.ctp.util.SQLWildcardUtil;

import org.apache.commons.lang.StringUtils;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.MatchMode;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;

import com.seeyon.ctp.common.dao.BaseHibernateDao;
import com.seeyon.ctp.organization.po.OrgUnit;
import com.seeyon.ctp.util.DBAgent;
import com.seeyon.ctp.util.Strings;
import com.seeyon.v3x.edoc.domain.EdocObjTeam;
import com.seeyon.v3x.edoc.domain.EdocObjTeamMember;

import org.hibernate.criterion.Restrictions;

public class EdocObjTeamDao extends BaseHibernateDao<EdocObjTeam> {
	
	public static final int PAGE_SIZE = 999;
	
	public List<EdocObjTeam> findAll()
	{
		List<EdocObjTeam> edocObjTeams=null;
		DetachedCriteria criteria = DetachedCriteria.forClass(EdocObjTeam.class)
		.add(Expression.eq("state",EdocObjTeam.STATE_USE));
		edocObjTeams=super.executeCriteria(criteria);
		return edocObjTeams;
	}
	
	public List<EdocObjTeam> findAllByAccount(Long accountId, boolean isPager)
	{
		List<EdocObjTeam> edocObjTeams=null;
		DetachedCriteria criteria = DetachedCriteria.forClass(EdocObjTeam.class)
				.add(Expression.eq("orgAccountId",accountId))
				.add(Expression.eq("state",EdocObjTeam.STATE_USE))
				.addOrder(Order.asc("sortId"))
				.addOrder(Order.desc("createTime"));
		;
		if(isPager){
			edocObjTeams=super.executeCriteria(criteria);
		}
		else{
			edocObjTeams=super.executeCriteria(criteria, -1, -1);
		}
		return edocObjTeams;
	}

	/**
	 * 根据条件分页查询机构组
	 * @param condition 查询条件
	 * @param flipInfo 分页参数
	 * @return
	 */
	public List<EdocObjTeam> findByConditionPager(Map<String,Object> condition, FlipInfo flipInfo)
	{
		List<EdocObjTeam> edocObjTeams=null;
		DetachedCriteria criteria = DetachedCriteria.forClass(EdocObjTeam.class)
				//.add(Expression.eq("orgAccountId",accountId))
				.add(Restrictions.eq("state",EdocObjTeam.STATE_USE))
				.addOrder(Order.asc("sortId"))
				.addOrder(Order.desc("createTime"));
		;
		if (condition!=null && !condition.isEmpty())
		{
			//机构id
			if (condition.containsKey("accountId")){
				criteria.add(Restrictions.eq("orgAccountId",ParamUtil.getLong(condition, "accountId")));
			}
			//名称
			if (condition.containsKey("name") && null!=condition.get("name") && Strings.isNotEmpty(condition.get("name").toString())){
				criteria.add(Restrictions.like("name",condition.get("name").toString(),MatchMode.ANYWHERE));
			}
		}
		if(flipInfo == null){
			flipInfo=new FlipInfo();
		}
		edocObjTeams=super.executeCriteria(criteria, flipInfo.getStartAt(), flipInfo.getSize());

		return edocObjTeams;
	}
	public List<EdocObjTeam> findAllSimpleByAccount(Long accountId, boolean isPager)
	{
	    List<Object[]> temp=null;
	    DetachedCriteria criteria = DetachedCriteria.forClass(EdocObjTeam.class)
	            .setProjection(Projections.projectionList().add(Projections.property("id")).add(Projections.property("name")).add(Projections.property("sortId")))
	            .add(Expression.eq("orgAccountId",accountId))
	            .add(Expression.eq("state",EdocObjTeam.STATE_USE))
	            .addOrder(Order.desc("createTime"));
	    ;
	    if(isPager){
	        temp=super.executeCriteria(criteria);
	    }
	    else{
	        temp=super.executeCriteria(criteria, -1, -1);
	    }
	    
	    List<EdocObjTeam> edocObjTeams = new ArrayList<EdocObjTeam>(temp.size());
	    for (Object[] o : temp) {
	        EdocObjTeam t = new EdocObjTeam();
	        t.setId((Long)o[0]);
	        t.setName((String)o[1]);
	        t.setSortId((Integer)o[2]);
	        
	        edocObjTeams.add(t);
        }
	    
	    return edocObjTeams;
	}
	
	
	public void updateState(String ids,Byte state)
	{
		String hql="update EdocObjTeam set state=? where id in (:ids)";
		Map<String,Object> namedParameter = new HashMap<String,Object>();
		List<Long> idList = new ArrayList<Long>();
		String[] tmp = ids.split(",");
		for(String id:tmp)
			idList.add(Long.valueOf(id));
		namedParameter.put("ids", idList);
		super.bulkUpdate(hql,namedParameter,state);
	}
	public EdocObjTeam findByAccountAndName(Long accountId,String orgName)
	{
		List<EdocObjTeam> edocObjTeams=null;
		DetachedCriteria criteria = DetachedCriteria.forClass(EdocObjTeam.class)
		.add(Expression.eq("orgAccountId",accountId))
		.add(Expression.eq("state",EdocObjTeam.STATE_USE))
		.add(Expression.eq("name",orgName))
		;
		edocObjTeams=super.executeCriteria(criteria);
		if(edocObjTeams.size()>0){return edocObjTeams.get(0);}
		else{return null;}
	}
	
	/**
	* @Title: getOrgTeamForDepartment
	* @Description: TODO 获取机构组下的部门信息
	* @param @param ids
	* @param @return    设定文件
	* @return Map<Long,String>    返回类型
	* @throws
	*/
	@SuppressWarnings("unchecked")
	public Map<Long,String> getOrgTeamForDepartment(String ids,Long loginAccountId){
		// TODO Auto-generated method stub
		Map<Long,String> map = new HashMap<Long,String>();
		if(StringUtils.isBlank(ids)){
			return map;
		}
		Set<Long> setValue = new HashSet<Long>();
		StringBuilder returnValue = new StringBuilder();
		
		List<OrgUnit> list = new ArrayList<OrgUnit>();
		List<EdocObjTeamMember>  edocObjTeamMemberlist = new ArrayList<EdocObjTeamMember>();
		
		Map<String,Object> params = new HashMap<String,Object>();
		StringBuilder hql = new StringBuilder();
		hql.append(" FROM OrgUnit as t , EdocObjTeamMember as u ").append(" WHERE t.id = u.memberId ").append(" AND  u.teamId in (:ids)");
		hql.append("  ORDER BY u.teamId,u.sortNum asc ");
		List<Long> idList = new ArrayList<Long>();
		String[] tmp = ids.split(",");
		for(String id:tmp){
			idList.add(Long.valueOf(id));
			if(idList.size() == PAGE_SIZE) {//sql中的in做分页处理
				params.put("ids", idList);
				List<Object[]> ll = DBAgent.find(hql.toString(), params);
				for (Object[] o : ll) {
					list.add((OrgUnit)o[0]);
					edocObjTeamMemberlist.add((EdocObjTeamMember)o[1]);
				}
				idList.clear();
			}
		}
		if(Strings.isNotEmpty(idList)) {
			params.put("ids", idList);
			List<Object[]> ll = DBAgent.find(hql.toString(), params);
			for (Object[] o : ll) {
				list.add((OrgUnit)o[0]);
				edocObjTeamMemberlist.add((EdocObjTeamMember)o[1]);
			}
		}
		
		Map<Long,Long> objTeamMemberMap = getEdocObjTeamMemberInfo(ids);
		int i = 0;
		int isAppend=0;
		int j=0;
		boolean isTrue=true;
		for(OrgUnit ou:list){
			if (!setValue.contains(objTeamMemberMap.get(edocObjTeamMemberlist.get(j).getId()))) {
				setValue.add(objTeamMemberMap.get(edocObjTeamMemberlist.get(j).getId()));
				if (isAppend >= 1) {
					returnValue.append("]");
					isAppend = 0;
					map.put(objTeamMemberMap.get(edocObjTeamMemberlist.get(j-1).getId()), returnValue.toString());
					returnValue.setLength(0);
					isTrue=false;
				}
				isAppend++;
				returnValue.append("[");
			}
			if(i++!=0&&isTrue){
				returnValue.append(",");
			}
			returnValue.append("'"+ou.getType()+"")
					   .append("|")
					   .append(ou.getOrgAccountId())
					   .append("_")
					   .append(ou.getId())
					   .append("|")
					   .append(ou.getName())
					   .append("'");
			isTrue=true;
			if (i == list.size()) {
				returnValue.append("]");
				isAppend = 0;
				map.put(objTeamMemberMap.get(edocObjTeamMemberlist.get(j).getId()), returnValue.toString());
				returnValue.setLength(0);
			}
			j++;
		}
		return map;
	
	}
	/**
	* @Title: getEdocObjTeamMemberInfo
	* @Description: TODO获取机构组关联表信息
	* @param @param ids
	* @param @return    设定文件
	* @return Map<Long,Long>    返回类型
	* @throws
	*/
	@SuppressWarnings("unchecked")
	private Map<Long,Long> getEdocObjTeamMemberInfo(String ids){
		Map<Long,Long> map = new HashMap<Long,Long>();
		List<EdocObjTeamMember> edocTeamMemberList = new ArrayList<EdocObjTeamMember>();
		
		Map<String,Object> params = new HashMap<String,Object>();
		StringBuilder hql=new StringBuilder();
		hql.append(" FROM EdocObjTeamMember ").append(" WHERE teamId IN(:ids)").append(" ORDER BY teamId asc");
		
		List<Long> idList = new ArrayList<Long>();
		String[] tmp = ids.split(",");
		for(String id:tmp){
			idList.add(Long.valueOf(id));
			if(idList.size() == PAGE_SIZE) {//sql中的in做分页处理
				params.put("ids", idList);
				List<Object> ll = DBAgent.find(hql.toString(), params);
				for (Object o : ll) {
					edocTeamMemberList.add((EdocObjTeamMember)o);
				}
				idList.clear();
			}
		}
		if(Strings.isNotEmpty(idList)) {
			params.put("ids", idList);
			List<Object> ll = DBAgent.find(hql.toString(), params);
			for (Object o : ll) {
				edocTeamMemberList.add((EdocObjTeamMember)o);
			}
		}
		
		for(EdocObjTeamMember objTeamMember : edocTeamMemberList){
			map.put(objTeamMember.getId(),objTeamMember.getTeamId());
		}
		return map;
	}


	/**
	 * @Title: 根据机构组id获取下级子数据
	 * @param @param id 机构组id
	 * @param @return    设定文件
	 * @return List<OrgUnit>   返回类型
	 * @throws
	 */
	@SuppressWarnings("unchecked")
	public List<OrgUnit> getOrgTeamChildrenByTeamId(Long id){
		List<OrgUnit> list = new ArrayList<OrgUnit>();
		if(null==id){
			return list;
		}
		Map<String,Object> params = new HashMap<String,Object>();
		StringBuilder hql = new StringBuilder();
		hql.append(" FROM OrgUnit as t , EdocObjTeamMember as u ").append(" WHERE t.id = u.memberId ").append(" AND  u.teamId = :id");
		hql.append("  ORDER BY u.teamId asc ");
		params.put("id", id);

		List<Object[]> ll = DBAgent.find(hql.toString(), params);
		for (Object[] o : ll) {
			list.add((OrgUnit)o[0]);
		}

		return list;

	}

	/**
	 * 根据查询条件分页获取机构组的子
	 * @param condition 条件
	 * @param flipInfo 分页参数
	 * @return
	 */
	public List<OrgUnit> getOrgTeamChildrenByConditionPager(Map<String,Object> condition, FlipInfo flipInfo){
		List<OrgUnit> list = new ArrayList<OrgUnit>();
		StringBuilder hql = new StringBuilder();
		Map<String,Object> parms=new HashMap<String, Object>(2);
		hql.append(" FROM OrgUnit as t , EdocObjTeamMember as u ").append(" WHERE t.id = u.memberId ");
		if (ParamUtil.getLong(condition,"teamId")!=null) {
			hql.append(" AND  u.teamId = :teamId");
			parms.put("teamId",ParamUtil.getLong(condition,"teamId"));
		}
		String name=ParamUtil.getString(condition,"name","");
		if (!"".equals(name)){
			hql.append(" AND  t.name like :name");
			parms.put("name","%" + SQLWildcardUtil.escape(name.trim()) + "%");
		}
		hql.append("  ORDER BY u.teamId asc ");

		List<Object[]> ll = DBAgent.find(hql.toString(),parms,flipInfo);
		for (Object[] o : ll) {
			list.add((OrgUnit)o[0]);
		}
		return list;

	}
	
}
